import 'dart:convert';
import 'dart:typed_data';

import 'tapers/registry.dart';
import 'tapers/taper.dart';

/// Entry point for many tape operations:
/// * TapeAdapters are registered here.
/// * It's a Codec turning objects into compressed tape bytes.
/// * It's a function that tapes objects. The counterpart is [untape].
/// * It's used as an annotation.
const tape = _TapeApi._();
Object untape(List<int> bytes) => tape.decode(bytes);

final _registry = Registry();
const _codec = adaptersCodec.fuse(blocksCodec);

class _TapeApi extends Codec<Object, List<int>> {
  const _TapeApi._();

  void register(Map<int, Taper<dynamic>> tapers) => _registry.register(tapers);

  @override
  Converter<Object, Uint8List> get encoder => throw UnimplementedError();

  @override
  Converter<List<int>, Object> get decoder => throw UnimplementedError();
}

// Namespaces for single tapers and groups of tapers.

/// Namespace for single tapers.
const taper = TaperApi._();

class TaperApi {
  const TaperApi._();
}

/// Namespace for groups of tapers.
const tapers = TapersApi._();

class TapersApi {
  const TapersApi._();
}
