import 'package:meta/meta.dart';

import '../adapters/adapters.dart';
import '../blocks/blocks.dart';

/// [TapeClassAdapter]s can be extended to support serializing and
/// deserializing Dart objects of type [T].
@immutable
abstract class TapeClassAdapter<T> extends Taper<T> {
  const TapeClassAdapter();

  Fields toFields(T object);
  T fromFields(Fields fields);

  @override
  T fromBlock(Block block) {
    final fields = Fields({
      for (final field in block.as<FieldsBlock>().fields.entries)
        field.key: adapters.decode(field.value),
    });
    return fromFields(fields);
  }

  @override
  Block toBlock(T object) {
    return FieldsBlock({
      for (final field in toFields(object)._fields.entries)
        field.key: adapters.encode(field.value),
    });
  }
}
