import 'dart:convert';

import 'registry.dart';

export 'errors.dart';
export 'registry.dart';
export 'taper.dart';
export 'utils.dart';

const adaptersCodec = _AdaptersCodec();

class _AdaptersCodec extends Codec<Object, Block> {
  const _AdaptersCodec();

  @override
  Converter<Object, Block> get encoder => const _AdaptersEncoder();

  @override
  Converter<Block, Object> get decoder => const _AdaptersDecoder();
}

class _AdaptersEncoder extends Converter<Object, Block> {
  const _AdaptersEncoder();

  @override
  Block convert(Object object) {
    final adapter = registry.valueToTaper(object);
    return TypedBlock(
      typeId: registry.taperToTypeCode(adapter),
      child: adapter.toBlock(object),
    );
  }
}

class _AdaptersDecoder extends Converter<Block, Object> {
  const _AdaptersDecoder();

  @override
  Object convert(Block block) {
    final typedBlock = block.as<TypedBlock>();
    return registry
        .typeCodeToTaper(typedBlock.typeId)
        .fromBlock(typedBlock.child);
  }
}
